
// FILE: src/config/config.js
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-image-uploader';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
	return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
	window.templateUrls.push({
		'package': packageName,
		'baseUrl': './bower_components/' + packageName + '/dist/templates/'
	});
}

// FILE: src/controllers/ImageUploaderController.js
function ImageUploaderController($scope, templateManager) {

	var self = this;

	var widgetAccessorDescriptor = {
		get: function() {
			return $scope.widget;
		}
	};

	var fieldAcessosDescriptor = {
		get: function() {
			return $scope.field;
		}
	};

	Object.defineProperty(self, "widget", widgetAccessorDescriptor);
	Object.defineProperty(self, "field", fieldAcessosDescriptor);

	/***************************************************
	 * Functions used by template
	 ***************************************************/
	this.getChooseFileText = function() {
		return self.field.chooseFileText || "Choose a file...";
	};

	this.getNoFileSelectedText = function() {
		return self.field.noFileSelectedText || "No file selected.";
	};

	this.getCurrentThumbnail = function() {
		var currentThumbnail = "//:0";
		if (typeof self.widget.currentRow[self.field.name] === "string") {
			var root = templateManager.project.fileServiceBaseUrl || "";
			currentThumbnail = root + self.widget.currentRow[self.field.name];
		} else if (Array.isArray(self.widget.currentRow[self.field.name]) && self.widget.currentRow[self.field.name].length > 0) {
			currentThumbnail = self.widget.currentRow[self.field.name][0].b64File;
		}

		return currentThumbnail;
	};

	this.isVisibleInCurrentFormTemplate = function() {};

	this.isEditMode = function() {
		return self.widget.mode === 'edit';
	};

	this.deselectFile = function(file) {
		self.widget.currentRow[self.field.name].splice(self.widget.currentRow[self.field.name].indexOf(file), 1);
		self.field.fileList = self.widget.currentRow[self.field.name];

		_callChangeEvent();
		_callAfterDeselectFileEvent();
	};

	function _callChangeEvent() {
		if (self.field.change) {
			self.field.change();
		}
	}

	function _callAfterDeselectFileEvent() {
		if (self.field.afterDeselectFile) {
			self.field.afterDeselectFile();
		}
	}
}

// FILE: src/directives/ImageUploaderDirective.js
(function() {
	var ZeedhiDirectives = angular.module('ZeedhiDirectives');
	ZeedhiDirectives.directive('zhImageUploader', zhImageUploader);

	function zhImageUploader(templateManager, ZHPromise) {

		var linkFunction = function(scope, element, attrs) {
			var container = element.parent();
			var field = scope.field;
			var widget = field.widget;
			var filesPromises = [];

			field.fileList = [];
			element.shouldNotPreventDefaultOnClick = true;
			element.parent().find("ul").nonbounce();
			element.on('change', onFileSelected);

			function onFileSelected() {
				var files = this.files;

				if (files.length > 0) {
					_ensureCurrentRowPropertyExistence();
					_setCurrentRowAsChanged();
					_putFilesIntoPromises(files);
					_executeAllPromises();
				}

				this.value = "";
			}

			function _ensureCurrentRowPropertyExistence() {
				widget.currentRow = widget.currentRow || {};
				widget.currentRow[field.name] = widget.currentRow[field.name] || [];
			}

			function _setCurrentRowAsChanged() {
				widget.currentRow.changed = true;
			}

			function _putFilesIntoPromises(files) {
				for (var i in files) {
					if (files.hasOwnProperty(i) && files[i] instanceof File) {
						filesPromises.push(_buildFileData(files[i]));
					}
				}
			}

			function _buildFileData(file) {
				var deferred = ZHPromise.defer();
				var fileReader = new FileReader();

				fileReader.onload = function(event) {
					widget.currentRow[field.name] = [_addExtraDataToFile(file)];
					field.fileList = widget.currentRow[field.name];
					deferred.resolve();
				};

				fileReader.readAsDataURL(file);
				return deferred;
			}

			function _addExtraDataToFile(file) {
				return Util.extend(file, {
					b64File: event.target.result
				}, true);
			}

			function _executeAllPromises() {
				ZHPromise.all(filesPromises)
					.then(function() {
						_callAfterSelectFileEvent();
						field.validate(field.value());
						templateManager.updateTemplate();
					});
			}

			function _callAfterSelectFileEvent() {
				if (field.afterSelectFile) {
					field.afterSelectFile();
				}
			}
		};

		return {
			restrict: 'A',
			link: linkFunction
		};
	}
})();

// FILE: src/registers/ImageUploaderRegister.js
function ImageUploaderRegister(DecodeValueFieldService){
    DecodeValueFieldService.registerType("zh-image-uploader#field/image-uploader.html", "image");
}

Configuration(function(ContextRegister) {
  ContextRegister.register('ImageUploaderRegister', ImageUploaderRegister);
});